## Copyright 2018 Augusta University
##
## Licensed under the Apache License, Version 2.0 (the "License");
## you may not use this file except in compliance with the License.
## You may obtain a copy of the License at
##
##    http://www.apache.org/licenses/LICENSE-2.0
##
## Unless required by applicable law or agreed to in writing, software
## distributed under the License is distributed on an "AS IS" BASIS,
## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
## See the License for the specific language governing permissions and
## limitations under the License.

include:
  - /formulas/{{ grains['role'] }}/install

{% import 'formulas/common/macros/constructor.sls' as constructor with context %}

/srv/salt:
  file.directory:
    - makedirs: true

/srv/addresses/addresses.db:
  file.managed:
    - replace: False
    - makedirs: True

compute_hosts:
  file.managed:
    - template: jinja
    - defaults:
        controller_hosts: {{ constructor.host_file_constructor(role='controller')|yaml_encode }}
        storage_hosts: {{ constructor.host_file_constructor(role='storage')|yaml_encode }}
        compute_hosts: {{ constructor.host_file_constructor(role='compute')|yaml_encode }}
        fpga_hosts: {{ constructor.host_file_constructor(role='fpga')|yaml_encode }}
        gpu_hosts: {{ constructor.host_file_constructor(role='gpu')|yaml_encode }}
        arm_hosts: {{ constructor.host_file_constructor(role='arm')|yaml_encode }}
    - names:
      - /etc/hosts:
        - source: salt://formulas/salt/files/hosts

addresses:
  sqlite3.table_present:
    - db: /srv/addresses/addresses.db
    - schema:
      - address TEXT UNIQUE
      - network TEXT
      - host TEXT
    - require:
      - file: /srv/addresses/addresses.db

{% for network in ['sfe', 'sbe', 'private'] %}
  {% for address in pillar['networking']['subnets'][network] | network_hosts %}
address_population_{{ address }}:
  sqlite3.row_present:
    - db: /srv/addresses/addresses.db
    - table: addresses
    - where_sql: address='{{ address }}'
    - data:
        address: {{ address }}
        network: {{ network }}
    - update: True
    - require:
      - sqlite3: addresses
  {% endfor %}
{% endfor %}

/srv/runners:
  file.directory:
    - makedirs: True

/srv/runners/needs.py:
  file.managed:
    - source: salt://_runners/needs.py

create_api_cert:
  cmd.run:
    - name: "salt-call --local tls.create_self_signed_cert"
    - creates:
      - /etc/pki/tls/certs/localhost.crt
      - /etc/pki/tls/certs/localhost.key

api:
  user.present:
    - password: {{ salt['pillar.get']('api:user_password', 'TBD') }}
    - hash_password: True

/etc/salt/master.d/gitfs_pillar.conf:
  file.managed:
    - contents: |
        ext_pillar:
          - git:
            - {{ pillar['kinetic_pillar_configuration']['branch'] }} {{ pillar['kinetic_pillar_configuration']['url'] }}:
              - env: base
        ext_pillar_first: False
        pillar_gitfs_ssl_verify: True

/etc/salt/master.d/gitfs_remotes.conf:
  file.managed:
    - contents: |
        gitfs_remotes:
          - {{ pillar['kinetic_remote_configuration']['url'] }}:
          {%- if salt['pillar.get']('kinetic_remote_configuration:token', False) %}
            - user: {{ pillar['kinetic_remote_configuration']['username'] }}
            - password: {{ pillar['kinetic_remote_configuration']['token'] }}
          {%- endif %}
            - saltenv:
              - base:
                - ref: {{ pillar['kinetic_remote_configuration']['branch'] }}
{% for remote, config in pillar.get('gitfs_other_configurations', {}).items() %}
          - {{ pillar['gitfs_other_configurations'][remote]['url'] }}:
          {%- if salt['pillar.get']('gitfs_other_configurations:'+remote+':token', False) %}
            - user: {{ pillar['gitfs_other_configurations'][remote]['username'] }}
            - password: {{ pillar['gitfs_other_configurations'][remote]['token'] }}
          {%- endif %}
            - saltenv:
              - base:
                - ref: {{ pillar['gitfs_other_configurations'][remote]['branch'] }}
{% endfor %}
        gitfs_saltenv_whitelist:
          - base

{% for directive, contents in pillar.get('master-config', {}).items() %}
/etc/salt/master.d/{{ directive }}.conf:
  file.managed:
    - contents_pillar: master-config:{{ directive }}
{% endfor %}

configure_netapi_clients:
  file.managed:
    - name: /etc/salt/master
    - source: salt://formulas/salt/files/master
    - user: salt
    - group: salt
    - mode: "0644"

/srv/dynamic_pillar:
  file.directory

/srv/dynamic_pillar/api.sls:
  file.managed:
    - replace: false
    - contents: |
        api:
          user_password: {{ salt['random.get_str']('64', punctuation=False) }}

/srv/dynamic_pillar/top.sls:
  file.managed:
    - source: salt://formulas/salt/files/top.sls
    - require:
      - file: /srv/dynamic_pillar/api.sls

/etc/kolla/config/:
  file.recurse:
    - source: salt://formulas/salt/files/config
    - include_empty: True
    - dir_mode: "0755"
    - file_mode: "0644"

/etc/kolla/globals.yml:
  file.managed:
    - source: salt://formulas/salt/files/globals.yml
    - user: root
    - group: root
    - mode: "0644"

/etc/kolla/config/keystone/domains/ldap_ca.crt:
  file.managed:
    - contents_pillar: ldap_ca

/etc/kolla/certificates/ca/ldap_ca.crt:
  file.managed:
    - contents_pillar: ldap_ca

/srv/dynamic_pillar/deps.sls:
  file.managed:
    - source: salt://formulas/salt/files/deps.sls
    - template: jinja
    - defaults:
{% if pillar['neutron']['backend'] == 'networking-ovn' %}
      ovsdb: "ovsdb: configure"
{% else %}
      ovsdb: ""
{% endif %}

salt-api_service:
  service.running:
    - name: salt-api
    - enable: True
    - watch:
      - file: /etc/salt/master
      - file: /etc/salt/master.d/*

build_phase_final:
  grains.present:
    - name: build_phase
    - value: configure

salt-master_watch:
  cmd.run:
    - name: 'salt-call service.restart salt-master'
    - bg: True
    - onchanges:
      - file: /etc/salt/master
      - file: /etc/salt/master.d/*
    - order: last
